#pragma once

#include <Windows.h>
#include <winternl.h>

// Function types
typedef VOID (NTAPI *PPS_APC_ROUTINE)(
    IN PVOID ApcArgument1,
    IN PVOID ApcArgument2,
    IN PVOID ApcArgument3
);

typedef VOID (*PKNORMAL_ROUTINE)(
    IN PVOID SystemArgument1,
    IN PVOID SystemArgument2,
    IN PVOID SystemArgument3
);

// Undocumented NT functions we need to import
typedef NTSTATUS (NTAPI *PFN_NT_QUEUE_APC_THREAD)(
    IN HANDLE ThreadHandle,
    IN PPS_APC_ROUTINE ApcRoutine,
    IN PVOID ApcArgument1 OPTIONAL,
    IN PVOID ApcArgument2 OPTIONAL,
    IN PVOID ApcArgument3 OPTIONAL
);

typedef NTSTATUS (NTAPI *PFN_NT_TEST_ALERT)(VOID);
typedef NTSTATUS (NTAPI *PFN_NT_ALERT_THREAD_BY_THREAD_ID)(IN HANDLE ThreadId);
typedef NTSTATUS (NTAPI *PFN_NT_WAIT_FOR_ALERT_BY_THREAD_ID)(IN HANDLE ThreadId, IN PLARGE_INTEGER Timeout OPTIONAL);
typedef NTSTATUS (NTAPI *PFN_ZW_YIELD_EXECUTION)(VOID);
typedef NTSTATUS (NTAPI *PFN_RTL_DISPATCH_EXCEPTION)(PEXCEPTION_RECORD ExceptionRecord, PCONTEXT ContextRecord);

// Improved SRW Lock structure
typedef struct _SLIM_LOCK {
    PVOID Ptr;                 // Signal state
    LIST_ENTRY WaitersList;    // List of waiting threads
    PVOID OwnerThread;         // Thread that owns the lock
    ULONG Flags;               // Lock state flags
} SLIM_LOCK, *PSLIM_LOCK;

// Enhanced thread state container
typedef struct _THREAD_CONTEXT {
    PSLIM_LOCK SlimLock;       // Pointer to thread's slim lock
    HANDLE ThreadId;           // Thread identifier
    HANDLE ThreadHandle;       // Thread handle for operations
    BOOLEAN IsWorker;          // Is this the worker thread?
    BOOLEAN HasWork;           // Does thread have pending work?
    PVOID WorkItem;            // Pointer to work item
    PVOID OriginalReturnAddress; // Original return address for routing
    PVOID RouteTarget;         // Where to route the thread
    PVOID ICallVector;         // CFG-validated function pointer
    PVOID StackAddress;        // Thread stack pointer for modification
} THREAD_CONTEXT, *PTHREAD_CONTEXT;

// Function declarations
BOOL InitializeSlimBloc();
VOID SmAlertThread(HANDLE ThreadHandle);
VOID NTAPI SmApc();
VOID SmService();
VOID SmService2();
VOID SmXcpt();
VOID SmWorker();
VOID SmStartup();
BOOL SmRouteThread(PTHREAD_CONTEXT ThreadCtx);
VOID RtlReleaseSRWLockExclusive(PSLIM_LOCK SlimLock);
BOOL IsThreadAlive(HANDLE ThreadHandle);
BOOL IsAddressInModule(PVOID Address, LPCSTR ModuleName);
VOID LogMessage(const char* Format, ...);
BOOL NtRvaToId(PVOID Address, PULONG ServiceId);
VOID CleanupThreadContexts();
VOID SmSignalEvent(HANDLE ThreadId);
PVOID FindReturnAddressOnStack(HANDLE ThreadHandle, PVOID StackPtr);

// For CFG bypass
PVOID SetupCfgBypass();
